;;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Base: 10 -*-
;;;; *************************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; ID:       $Id: assert.lisp,v 1.4 2003/08/04 16:13:58 kevin Exp $
;;;; Purpose:  Assert functions for XLUnit
;;;;
;;;; *************************************************************************

(in-package #:xlunit)


(define-condition assertion-failed (simple-condition) 
  ((msg :initform nil :initarg :msg :accessor msg))
  (:documentation "Base class for all test failures."))


(defun failure-msg (msg &optional format-str &rest args)
  "Signal a test failure and exit the test."
  (signal 'assertion-failed
	  :msg msg
	  :format-control format-str
	  :format-arguments args))

(defun failure (format-str &rest args)
  "Signal a test failure and exit the test."
  (apply #'failure-msg nil format-str args))

(defun assert-equal (v1 v2 &optional msg)
  (unless (equal v1 v2)
    (failure-msg msg "Test equal: ~S ~S" v1 v2)))

(defun assert-eql (v1 v2 &optional msg)
  (unless (eql v1 v2)
    (failure-msg msg "Test eql: ~S ~S" v1 v2)))

(defmacro assert-true (v &optional msg)
  `(unless ,v
    (failure-msg msg "Not true: ~S" ',v)))

(defmacro assert-false (v &optional msg)
  `(when ,v
     (failure-msg msg "Not false: ~S" ',v)))
